unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Registry, StdCtrls, Buttons, FileCtrl, ComCtrls, Gauges;

type
  TfrmMain = class(TForm)
    PageControl1: TPageControl;
    Tab2: TTabSheet;
    Tab1: TTabSheet;
    Label1: TLabel;
    Label2: TLabel;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    Bar: TGauge;
    lblProgress: TLabel;
    edtSourceFolder: TEdit;
    edtTargetFolder: TEdit;
    Memo: TMemo;
    btnStart: TButton;
    btnClose: TButton;
    Button1: TButton;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure SpeedButtonClick(Sender: TObject);
    procedure btnStartClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private

  public

  end;

const
  smfh: array[0..11] of byte = (
    $66, $73, $73, $6D, $01, $00, $00, $00, $01, $00, $00, $00
  );

var
  frmMain: TfrmMain;
  AbortAll: Boolean;
  wav2Dbsc: array[0..117] of byte = (
    $73, $62, $68, $73, $08, $00, $00, $00, $01, $06, $00, $00, $00, $4E, $6F, $72,
    $6D, $61, $6C, $00, $00, $00, $00, $01, $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00,
    $00, $00, $00, $06, $00, $00, $00, $3C, $6E, $6F, $6E, $65, $3E, $01, $00, $00,
    $00, $00, $00, $00, $00, $01, $00, $00, $00, $10, $00, $00, $00, $44, $65, $66,
    $61, $75, $6C, $74, $3A, $3A, $44, $65, $66, $61, $75, $6C, $74, $00, $00, $80,
    $3F, $00, $00, $80, $3F, $00, $00, $80, $3F, $00, $00, $00, $00, $00, $00, $00,
    $00, $01, $07, $00, $00, $00
  );

implementation

{$R *.dfm}

procedure TfrmMain.FormCreate(Sender: TObject);
var
  CohDir: string;
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKey('\SOFTWARE\THQ\Company of Heroes', True) then
    begin
      CohDir := Reg.ReadString('InstallDir');

      if DirectoryExists(CohDir + 'ww2\data\sound\wav\') then edtSourceFolder.Text := CohDir + 'ww2\data\sound\wav\'
      else if DirectoryExists(CohDir) then edtSourceFolder.Text := CohDir
      else edtSourceFolder.Text := 'C:\';

      Reg.CloseKey;
    end;
  finally
   Reg.Free;
  end;

  PageControl1.TabIndex := 0;
end;

procedure TfrmMain.SpeedButtonClick(Sender: TObject);
var
  Folder: string;
begin
  if SelectDirectory('Select folder for .smf files:', '', Folder) then
  begin
    if Folder[Length(Folder)] <> '\' then Folder := Folder + '\';

    if TSpeedButton(Sender).Name = 'SpeedButton1' then edtSourceFolder.Text := Folder
    else edtTargetFolder.Text := Folder;
  end;
end;

procedure SearchFiles(Path, Filename: string; TargetList: TStrings);
var
  Rec: TSearchRec;
begin
  if FindFirst(Path + Filename, faAnyFile - faDirectory, Rec) = 0 then
  try
    repeat
      Application.ProcessMessages;
      if AbortAll then Exit;
      TargetList.Add(Path + Rec.Name);
    until FindNext(Rec) <> 0;
  finally
    FindClose(Rec);
  end;

  if FindFirst(Path + '*.*', faDirectory, Rec) = 0 then
  try
    repeat
      Application.ProcessMessages;
      if AbortAll then Exit;
      if ((Rec.Attr and faDirectory) <> 0) and (Rec.Name <> '.') and (Rec.Name <> '..') then
        SearchFiles(Path + Rec.Name + '\', Filename, TargetList);
    until FindNext(Rec) <> 0;
  finally
    FindClose(Rec);
  end;
end;

procedure TfrmMain.btnStartClick(Sender: TObject);
var
  I: Integer;
  Files: TStringlist;
  ReadStream, WriteStream: TMemoryStream;
  Filetype: Char;
  Filename, S, S2, Skipped: string;
begin
  if not DirectoryExists(edtTargetFolder.Text) then
  begin
    Showmessage('Select target folder first.');
    Exit;
  end;

  Screen.Cursor :=  crHourGlass;
  AbortAll := False;
  btnClose.Caption := 'Abort';
  SpeedButton1.Enabled := False;
  SpeedButton2.Enabled := False;
  btnStart.Enabled := False;
  Memo.Clear;
  Memo.Lines.Add('Searching .smf files...'#13#10);
  Files := TStringlist.Create;
  ReadStream := TMemoryStream.Create;
  WriteStream := TMemoryStream.Create;
  SearchFiles(edtSourceFolder.Text, '*.smf', Files);

  if Files.Count = 0 then
  begin
    Memo.Lines.Add('No .smf files found.'#13#10);
  end
  else
  begin
    lblProgress.Caption := '';
    lblProgress.Visible := True;
    Bar.Progress := 0;
    Bar.MinValue := 0;
    Bar.MaxValue := Files.Count - 1;
    Bar.Visible := True;
    Memo.Lines.Add('Starting...'#13#10);

    Skipped := '';
    for I := 0 to Files.Count - 1 do
    try
      if AbortAll then Break;

      lblProgress.Caption := 'File ' + IntToStr(I + 1) + '/' + IntToStr(Files.Count);
      ReadStream.LoadFromFile(Files[I]);
      ReadStream.Seek(12, soFromBeginning);
      ReadStream.Read(Filetype, 1);
      ReadStream.Seek(12, soFromBeginning);
      WriteStream.Clear;
      WriteStream.CopyFrom(ReadStream, ReadStream.Size - 12);  

      Filename := Copy(Files[I], Length(edtSourceFolder.Text) + 1, Length(Files[I]));
      // Very simple filetype check
      if Filetype = 'R' then Filename := ChangeFileExt(Filename, '.wav') else Filename := ChangeFileExt(Filename, '.mp3');

      // Create missing folders
      S := Filename;
      S2 := edtTargetFolder.Text;
      while not DirectoryExists(ExtractFilePath(edtTargetFolder.Text + Filename)) do
      begin
        S2 := S2 + Copy(S, 1, Pos('\', S));
        if not DirectoryExists(S2) then CreateDir(S2);

        if Pos('\', S) > 0 then Delete(S, 1, Pos('\', S))
        else Break;
      end;

      WriteStream.SaveToFile(edtTargetFolder.Text + Filename);

      Bar.AddProgress(1) ;
      Memo.Lines.Add(Filename);
      Application.ProcessMessages;
    except
      Skipped := Skipped + #13#10 + Files[I];
    end;

    Memo.Lines.Add(#13#10'Finished.');

    if Skipped <> '' then Memo.Lines.Add(#13#10#13#10'Files skipped: ' + #13#10 + Skipped);
  end;

  Bar.Hide;
  lblProgress.Visible := False;
  SpeedButton1.Enabled := True;
  SpeedButton2.Enabled := True;
  btnStart.Enabled := True;
  btnClose.Caption := 'Close';
  Screen.Cursor :=  crDefault;
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
  if TButton(Sender).Caption = 'Abort' then AbortAll := True else Close;
end;

procedure TfrmMain.Button1Click(Sender: TObject);
var
  FN: string;
  FL: Integer;
  ReadStream, WriteStream: TMemoryStream;
begin
  if not OpenDialog.Execute then Exit;

  SaveDialog.FileName := ChangeFileExt(ExtractFilename(OpenDialog.FileName), '.bsc');
  if not SaveDialog.Execute then Exit;

  ReadStream := TMemoryStream.Create;
  WriteStream := TMemoryStream.Create;

  ReadStream.LoadFromFile(OpenDialog.FileName);
  WriteStream.Write(smfh, 12);
  WriteStream.CopyFrom(ReadStream, ReadStream.Size);
  WriteStream.SaveToFile(ChangeFileExt(ExtractFilename(SaveDialog.FileName), '.smf'));

  FN := ChangeFileExt(ExtractFilename(SaveDialog.FileName), '');
  FL := Length(FN);
  WriteStream.Clear;
  WriteStream.Write(wav2Dbsc, 118);
  WriteStream.Write(PChar(FN)^, Length(FN));
  WriteStream.Seek(114, soFromBeginning);
  WriteStream.Write(FL, 1);
  WriteStream.SaveToFile(SaveDialog.FileName);

  MessageDlg('Files saved.', mtInformation, [mbOK], 0);
end;

end.
